#ifndef EETRESULT_H
#define EETRESULT_H

#include <QObject>
#include <QUuid>
#include <QString>
#include <QList>
#include <QDateTime>

class EetMessage : public QObject
{
    Q_OBJECT
public:
    explicit EetMessage(QObject *parent = 0);
    EetMessage(int code, const QString &message, QObject *parent = 0);

    int code() const;
    void setCode(int code);

    QString message() const;
    void setMessage(const QString &message);

private:
    int m_code;
    QString m_message;
};

typedef QList<EetMessage*> EetMessageList;

class EetResult : public QObject
{
    Q_OBJECT
public:
    enum ResponseStatus
    {
        RESPONSE_OK = 0,
        DATA_ERROR,
        USER_CERT_ERROR,
        SERVER_ERROR,
        SERVICE_CERT_ERROR,
        INVALID_SIGNATURE,
        SSL_ERROR,
        TIMEDOUT,
        OFFLINE
    };

    explicit EetResult(QObject *parent = 0);

    ResponseStatus status() const;
    void setStatus(const ResponseStatus &status);

    QString fik() const;
    void setFik(const QString &fik);

    EetMessageList warnings() const;
    void setWarnings(const EetMessageList &warnings);

    EetMessageList errors() const;
    void setErrors(const EetMessageList &errors);

    QUuid uuid() const;
    void setUuid(const QUuid &uuid);

    QDateTime reciveDate() const;
    void setReciveDate(const QDateTime &reciveDate);

    QString pkp() const;
    void setPkp(const QString &pkp);

    QString bkp() const;
    void setBkp(const QString &bkp);

private:
    ResponseStatus m_status;
    QUuid m_uuid;
    QDateTime m_reciveDate;
    QString m_fik;
    QString m_pkp;
    QString m_bkp;
    EetMessageList m_warnings;
    EetMessageList m_errors;
};

#endif // EETRESULT_H
